package com.app.security

import groovy.transform.CompileStatic
import org.springframework.context.MessageSource
import com.app.security.auth.LoginDataService

import static org.springframework.security.acls.domain.BasePermission.ADMINISTRATION
import static org.springframework.security.acls.domain.BasePermission.DELETE
import static org.springframework.security.acls.domain.BasePermission.READ

import org.springframework.dao.DataAccessException
import org.springframework.security.acls.domain.PermissionFactory
import org.springframework.security.acls.domain.PrincipalSid
import org.springframework.security.acls.model.Permission
import org.springframework.security.acls.model.Sid
import org.springframework.security.core.Authentication

import grails.plugin.springsecurity.SpringSecurityService
import grails.plugin.springsecurity.acl.AclUtilService
import grails.plugin.springsecurity.annotation.Secured

@CompileStatic
@Secured(['ROLE_USER'])
class SecureController {

	static allowedMethods = [index: 'GET',
							 create: 'GET',
							 add: 'POST',
							 del: 'POST',
							 addPermission: 'POST',
							 adminPermission: 'GET',
							 deletePermission: 'POST',
							 debug: 'GET',
	]

	private static final Permission[] HAS_DELETE = [DELETE, ADMINISTRATION]
	private static final Permission[] HAS_ADMIN = [ADMINISTRATION]

	PermissionFactory aclPermissionFactory
	AclUtilService aclUtilService
	LoginService contactService
	SpringSecurityService springSecurityService
	MessageSource messageSource
	LoginDataService contactDataService

	def index() {
		List<Login> myContactsList = contactService.findAll()
		Map<Login, Boolean> hasDelete = [:]
		Map<Login, Boolean> hasAdmin = [:]

		Authentication user = springSecurityService.authentication
		for (Login contact in myContactsList) {
			hasDelete[contact] = aclUtilService.hasPermission(user, contact, HAS_DELETE)
			hasAdmin[contact] = aclUtilService.hasPermission(user, contact, HAS_ADMIN)
		}

		[contacts: myContactsList, hasDeletePermission: hasDelete, hasAdminPermission: hasAdmin]
	}


	def create() {
		[command: new SaveLoginCommand()]
	}

	def add(SaveLoginCommand cmd) {
		if (cmd.hasErrors()) {
			return [command: cmd]
		}

		contactService.create cmd.name, cmd.email

		redirect action: 'index'
	}

	def del(Long contactId) {
		Login contact = contactService.findById(contactId)
		contactService.delete contact
		[contact: contact]
	}


	def adminPermission(Long contactId) {
		Login contact = contactService.findById(contactId)
		[contact: contact, acl: aclUtilService.readAcl(contact)]
	}

	def addPermission(AddPermissionCommand addPermission) {

		if ( addPermission.hasErrors() ) {
			if ( !addPermission.contactId ) {
				flash.message = 'Contact id is required'
			}
			redirect action: 'index'
			return
		}

		Login contact = contactService.findById(addPermission.contactId)

		def buildModel = { ->
			[command: addPermission, contact: contact,
			 recipients: contactService.allRecipients, permissions: listPermissions()]
		}

		if (request.get) {
			return buildModel()
		}

		if (request.post) {
			if (addPermission.hasErrors()) {
				return buildModel()
			}

			PrincipalSid sid = new PrincipalSid(addPermission.recipient)
			Permission permission = aclPermissionFactory.buildFromMask(addPermission.permission)

			try {
				contactService.addPermission contact, sid, permission
				redirect action: 'index'
			}
			catch (DataAccessException existingPermission) {
				log.error existingPermission.message, existingPermission
				addPermission.errors.rejectValue 'recipient', 'err.recipientExistsForContact'
				return buildModel()
			}
		}
	}


	def deletePermission(Long contactId, String sid, Integer permission) {

		Login contact = contactService.findById(contactId)
		Sid sidObject = new PrincipalSid(sid)
		Permission p = aclPermissionFactory.buildFromMask(permission)

		contactService.deletePermission contact, sidObject, p

		[contact: contact, sid: sidObject, permission: p]
	}

	def debug() {
		[auth: springSecurityService.authentication]
	}

	private Map<Integer, String> listPermissions() {
		[
				(ADMINISTRATION.mask): messageSource.getMessage('select.administer', [] as Object[], 'Admnistration', request.locale),
		 		(READ.mask): messageSource.getMessage('select.read', [] as Object[], 'Admnistration', request.locale),
		 		(DELETE.mask): messageSource.getMessage('select.delete', [] as Object[], 'Admnistration', request.locale)
		] as Map<Integer, String>
	}
}