package com.app.security

import grails.gorm.transactions.ReadOnly
import grails.gorm.transactions.Transactional
import grails.plugin.springsecurity.acl.AclClass
import grails.plugin.springsecurity.acl.AclEntry
import org.springframework.security.access.prepost.PostFilter
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.acls.domain.BasePermission
import org.springframework.security.acls.domain.PrincipalSid
import org.springframework.security.acls.model.AccessControlEntry
import org.springframework.security.acls.model.MutableAcl
import org.springframework.security.acls.model.Permission
import org.springframework.security.acls.model.Sid
import grails.plugin.springsecurity.SpringSecurityService
import grails.plugin.springsecurity.acl.AclService
import grails.plugin.springsecurity.acl.AclUtilService
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import com.app.security.auth.LoginDataService
import com.app.security.auth.LoginUserDataService

@CompileStatic
@Slf4j
class LoginService {

	AclService aclService
	AclUtilService aclUtilService
	SpringSecurityService springSecurityService
	LoginDataService contactDataService
	LoginUserDataService userDataService

	@PreAuthorize('hasPermission(#contact, admin)')
	@Transactional
	void addPermission(Login contact, Sid recipient, Permission permission) {
		aclUtilService.addPermission Login, contact.id, recipient, permission
	}

	@Transactional
	protected void addPermission(Login contact, Permission permission) {
		aclUtilService.addPermission Login, contact.id, loggedUsername(), permission
	}

	protected String loggedUsername() {
		springSecurityService.authentication.name
	}

	@PreAuthorize('hasRole("ROLE_USER")')
	Login create(String name, String email) {
		Login contact = contactDataService.save(name, email)
		if ( contact.hasErrors() ) {
			log.error 'Error while saving contact'
			return contact
		}

		addPermission(contact, BasePermission.ADMINISTRATION)

		log.debug "Created contact $contact and granted admin permission to recipient ${loggedUsername()}"
		contact
	}

	@PreAuthorize('hasPermission(#contact, "delete") or hasPermission(#contact, admin)')
	@Transactional
	void delete(Login contact) {
		Long id = contact.id
		contactDataService.delete(id)

		// Delete the ACL information as well
		aclUtilService.deleteAcl contact

		log.debug "Deleted contact $contact including ACL permissions"
	}

	@PreAuthorize('hasPermission(#contact, admin)')
	@Transactional
	void deletePermission(Login contact, Sid recipient, Permission permission) {
		MutableAcl acl = (MutableAcl)aclUtilService.readAcl(contact)

		// Remove all permissions associated with this particular recipient (string equality to KISS)
		acl.entries.eachWithIndex { AccessControlEntry entry, int i ->
			if (entry.sid == recipient && entry.permission == permission) {
				acl.deleteAce i
			}
		}

		aclService.updateAcl acl

		log.debug "Deleted contact $contact ACL permissions for recipient $recipient"
	}

	@PreAuthorize('hasRole("ROLE_USER")')
	@PostFilter('hasPermission(filterObject, "read") or hasPermission(filterObject, admin)')
	@ReadOnly
	List<Login> findAll() {
		log.debug 'Returning all contacts'
		List<Login> contactList = contactDataService.findAllOrderById()
		contactList
	}

	@PreAuthorize('hasRole("ROLE_USER")')
	@ReadOnly
	List<String> getAllRecipients() {
		log.debug 'Returning all recipients'
		userDataService.findLoginUserUsername()
	}

	@PreAuthorize('hasPermission(#id, "com.app.security.Login", read) or hasPermission(#id, "com.app.security.Login", admin)')
	@ReadOnly
	Login findById(Long id) {
		log.debug "Returning contact with id: $id"
		contactDataService.findById(id)
	}

	@ReadOnly
	Login findRandomContact() {
		log.debug 'Returning random contact'
		List<Long> ids = contactDataService.findLoginId()
		Long id = ids[new Random().nextInt(ids.size())]
		contactDataService.findById(id)
	}

	@Transactional
	void update(Login contact, String email, String name) {
		Serializable id = contact.id
		contactDataService.update(id, name, email)
		log.debug "Updated contact $contact"
	}
}
